#!/usr/bin/env python3
import argparse
import qepy

QEBINS = {'alpha2f.x': 'alpha2f',
 'average.x': 'average',
 'bands.x': 'do_bands',
 'benchmark_libxc.x': 'benchmark_libxc',
 'cell2ibrav.x': 'cell2ibrav',
 'dist.x': 'pwscf',
 'dos.x': 'do_dos',
 'dynmat.x': 'dynmat',
 'epa.x': 'epa',
 'epsilon.x': 'epsilon',
 'ev.x': 'ev',
 'fd.x': 'fd',
 'fd_ef.x': 'fd_raman',
 'fd_ifc.x': 'fd_ifc',
 'fermi_proj.x': 'fermi_proj',
 'fermi_velocity.x': 'fermi_velocity',
 'fqha.x': 'fqha',
 'fs.x': 'fermisurface',
 'gww.x': 'gww',
 'gww_fit.x': 'gww_fit',
 'hp.x': 'hp_main',
 'ibrav2cell.x': 'ibrav2cell',
 'initial_state.x': 'initial_state',
 'kpoints.x': 'special_points',
 'lambda.x': 'elph',
 'ld1.x': 'ld1',
 'manypw.x': 'pwscf',
 'matdyn.x': 'matdyn',
 'molecularpdos.x': 'molecularpdos',
 'neb.x': 'neb',
 'open_grid.x': 'open_grid',
 'path_interpolation.x': 'images_interpolator',
 'ph.x': 'phonon',
 'plan_avg.x': 'plan_avg',
 'plotband.x': 'plotband',
 'plotproj.x': 'plotproj',
 'plotrho.x': 'plotrho',
 'pmw.x': 'pmw',
 'pp.x': 'pp',
 'ppacf.x': 'do_ppacf',
 'projwfc.x': 'do_projwfc',
 'pw.x': 'pwscf',
 'pw2bgw.x': 'pw2bgw',
 'pw2critic.x': 'pw2critic',
 'pw2gw.x': 'pw2gw',
 'pw2wannier90.x': 'pw2wannier90',
 'pw4gww.x': 'gwl_punch',
 'pwcond.x': 'pwcond',
 'q2qstar.x': 'Q2QSTAR',
 'q2r.x': 'q2r',
 'q2trans.x': 'q2trans',
 'q2trans_fd.x': 'q2trans_fd',
 'simple.x': 'simple',
 'simple_bse.x': 'simple_bse',
 'simple_ip.x': 'simple_ip',
 'sumpdos.x': 'sumpdos',
 'turbo_davidson.x': 'lr_dav_main',
 'turbo_eels.x': 'lr_eels_main',
 'turbo_lanczos.x': 'lr_main',
 'turbo_spectrum.x': 'lr_calculate_spectrum',
 'wannier_ham.x': 'wannier_ham',
 'wannier_plot.x': 'wannier_plot',
 'wfck2r.x': 'wfck2r'}

def get_parse(prog = 'pw.x', parser = None):
    if parser is None :
        parser = argparse.ArgumentParser(description=prog)
    parser.add_argument(f'--{prog}', dest=prog, action='store_true',
            default=False, help=f'Same as regular "{prog}" in QE.')
    return parser

def get_args(prog):
    parser = get_parse(prog)
    args, others = parser.parse_known_args()
    args.qecmd = ' '.join(others)
    qepy.qepy_sys.set_command_line(args.qecmd)
    return args

def run(args, prog):
    job = getattr(qepy, QEBINS[prog])
    print(job)
    job()

def main(prog):
    args = get_args(prog)
    run(args, prog)



import os
import types
from functools import wraps


class Logger(type):
    def __new__(cls, name, bases, attrs):

        for attr_name, attr_value in attrs.items():
            if isinstance(attr_value, types.FunctionType):
                attrs[attr_name] = cls.stdout2file(attr_value)

        return super().__new__(cls, name, bases, attrs)

    @staticmethod
    def stdout2file(function, fileobj = None):
        return stdout2file(function, fileobj=fileobj)

    @staticmethod
    def print2file(fileobj = None):
        return print2file(fileobj=fileobj)

def print2file(fileobj = None):
    def decorator(function):
        return stdout2file(function, fileobj=fileobj)
    return decorator

def stdout2file(function, fileobj = None):
    @wraps(function)
    def wrapper(*args, **kwargs):
        fobj = fileobj
        if not fobj :
            if len(args) > 0 and hasattr(args[0], 'fileobj'): fobj = args[0].fileobj
        stdout = None
        if fobj is not None and not fobj.closed :
            if os.fstat(1).st_ino != os.fstat(fobj.fileno()).st_ino :
                stdout = os.dup(1)
                os.dup2(fobj.fileno(), 1)
        results = function(*args, **kwargs)
        if stdout is not None :
            os.dup2(stdout, 1)
            os.close(stdout)
        return results
    return wrapper


env = {
    'STDOUT' : None,  # file descriptor of output
    'DRIVER' : None, # save the instance of driver class
}



# Fix
# MPI_IN_PLACE and MKL
import sys, os
from ctypes import util, CDLL, RTLD_LOCAL, RTLD_GLOBAL
if 'mpi4py' in sys.modules :
    if hasattr(util, '_findLib_ld') and hasattr(util, '_get_soname') :
        mpilib = util._get_soname(util._findLib_ld('mpi'))
    else :
        mpilib = None
    mpilib = mpilib or util.find_library('mpi') or util.find_library('mpifort')
    try:
        CDLL(mpilib, RTLD_LOCAL | RTLD_GLOBAL)
    except Exception :
        pass
try:
    if hasattr(util, '_findLib_ld'):
        mkllib = os.path.basename(util._findLib_ld('mkl_rt'))
    else :
        mkllib = util.find_library('mkl_rt')
    CDLL(mkllib, RTLD_LOCAL | RTLD_GLOBAL)
except Exception :
    pass

# control the output
import types
from .core import Logger, env
class QEpyLib :
    def __init__(self, **kwargs):
        import _qepy as qepylib
        sys.modules['_qepy'] = self
        self.qepylib =qepylib

    def __getattr__(self, attr):
        attr_value = getattr(self.qepylib, attr)
        if '__array__' not in attr :
            attr_value = Logger.stdout2file(attr_value, fileobj=env['STDOUT'])
        return attr_value
qepylib = QEpyLib()
# End fix



# import atexit

# def pwscf_finalise():
#     qepy_pwscf_finalise()


# atexit.register(pwscf_finalise)

import pkgutil
import operator
def qepy_clean_saved():
    mods = [name for _, name, _ in pkgutil.iter_modules(qepy.__path__)]
    for mod in mods :
        if hasattr(qepy, mod):
            for item in ['_arrays', '_objs'] :
                if hasattr(operator.attrgetter(mod)(qepy), item):
                    attr = mod + '.' + item
                    operator.attrgetter(attr)(qepy).clear()


qepy_clean_saved()
__author__ = "Pavanello Research Group"
__contact__ = "m.pavanello@rutgers.edu"
__version__ = "6.5.0rc0"
__license__ = "GPL"
__date__ = "2023-09-04"

try:
    from importlib.metadata import version # python >= 3.8
except Exception :
    try:
        from importlib_metadata import version
    except Exception :
        pass

try:
    __version__ = version("qepy")
except Exception :
    pass

from qepy.driver import Driver



#!/usr/bin/env python3
import argparse
from qepy.cui import qex

def get_args():
    parser = argparse.ArgumentParser(description='QEpy tasks :')
    #-----------------------------------------------------------------------
    for prog in qex.QEBINS :
        parser = qex.get_parse(prog=prog, parser=parser)
    #-----------------------------------------------------------------------
    args = parser.parse_args()
    return args

def main():
    import sys
    for prog in qex.QEBINS :
        key = '--' + prog
        if key in sys.argv :
            return qex.main(prog)
    else :
        get_args()
